#ifndef HIERARCHICAL_CHANNEL_H_INCLUDED_
#define HIERARCHICAL_CHANNEL_H_INCLUDED_

#include <vector>

#include "channel.h"
#include "rose.h"

#include "function.h"
#include "thread.h"
#include "cthread.h"
#include "method.h"

namespace risc {

class HierarchicalChannel: public Channel{

public:

  ~HierarchicalChannel();

  /**
   * \brief Default constructor
   */
  explicit HierarchicalChannel(SgClassDefinition *ast_node);

  /**
   * \brief  This function searches for all functions, SC_THREADS, SC_CTHREADS,
   *         and SC_METHODS  in the module definition. The corresponding
   *         variable vectors will be used.
   */
  virtual void initialize_functions();

  /**
   * \brief Vector of threads which are defined in this module
   */
  ThreadVector threads_;

  /**
   * \brief Vector of cthreads which are defined in this module
   */
  CThreadVector cthreads_;

  /**
   * \brief Vector of methods which are defined in this module
   */
  MethodVector methods_;

  /**
   * \brief Vector of reset signals which are registered in this module
   */
  ResetSignalVector reset_signals_;

  /**
   * \brief Vector of async reset signals which are registered in this module
   */
  ResetSignalVector async_reset_signals_;

  /**
   * \brief This variable counts how many instances of this given type exsist
   *        in the desgin
   */
  int instance_counter_;

private:
  /**
   * \brief Disabled copy constructor
   */
  HierarchicalChannel(const HierarchicalChannel &hcd);
};

typedef std::vector<HierarchicalChannel*>
            HierarchicalChannelVector;
typedef std::vector<HierarchicalChannel*>::iterator
            HierarchicalChannelVectorIter;
typedef std::vector<HierarchicalChannel*>::const_iterator
            HierarchicalChannelVectorConstIter;

}; // end of namespace risc

#endif /* HIERARCHICAL_CHANNEL_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
