#include "instance_tree.h"

#include "design.h"
#include "event.h"
#include "instance.h"
#include "inport.h"
#include "inoutport.h"
#include "outport.h"
#include "hierarchical_channel.h"
#include "hierarchical_channel_instance.h"
#include "primitive_channel_instance.h"

#include "../tools/misc.h"
#include "../tools/variable_helper.h"

risc::InstanceTree::InstanceTree()
{ }

risc::InstanceTree::InstanceTree(const InstanceTree &other)
{
  path_ = other.path_;
}

risc::InstanceTree
risc::InstanceTree::get_real_mapped_variable()
{
  InstanceTree mapped_variable;

  Element element = path_.back();
  Object *object = element.second;

  // we only consider references, so we sort all other out
  if(!object->is_reference() && !object->is_pointer()) {
    return *this;
  } else {

    InstanceTree base = *this;
    InstanceTree parent = get_parent();
    SgVariableSymbol *base_variable_symbol = NULL;
    SgVariableSymbol *parent_variable_symbol = NULL;

    base_variable_symbol = object->get_symbol();

    // basically, we are iterating as long until we get the first symbol
    // which is not a reference
    do {

      if(!parent.path_.empty()) {

        Element element = base.path_.back();

        parent_variable_symbol = get_mapped_reference(
            base_variable_symbol,
            element.first->get_ast_node(),
            parent.get_scope()->get_ast_node(),
            dynamic_cast<SgVariableDefinition*>(
              parent.get_object()->get_ast_node()));

        base = parent;
        parent = parent.get_parent();

        base_variable_symbol = parent_variable_symbol;
      }
    } while(isSgReferenceType(base_variable_symbol->get_type()));

    mapped_variable = base;

    if(dynamic_cast<Variable*>(object)) {

      mapped_variable.path_.back().second
        = base.get_scope()->find_variable(
            isSgVariableDeclaration(base_variable_symbol->get_declaration()
              ->get_declaration())->get_definition());

      // we consider this case for global variables
      if(mapped_variable.path_.back().second == NULL) {


        mapped_variable.path_.clear();
        Element e(NULL, risc::get_design()->find_global_variable(
              isSgVariableDeclaration(base_variable_symbol->get_declaration()
                ->get_declaration())->get_definition()));


        mapped_variable.path_.push_back(e);
      }
    }

    if(dynamic_cast<Event*>(object)) {

      mapped_variable.path_.back().second
        = base.path_.back().first->find_event(
            isSgVariableDeclaration(base_variable_symbol->get_declaration()
              ->get_declaration())->get_definition()) ;
    }
  }

  return mapped_variable;
}

SgVariableSymbol*
risc::InstanceTree::get_mapped_reference(
    SgVariableSymbol *reference,
    SgClassDefinition *base_class_def,
    SgClassDefinition *parent_class_def,
    SgVariableDefinition *parent_member)
{
  if(!tools::is_function_parameter_variable(reference) ) {

    SgMemberFunctionDeclaration *ctor_base
      = risc::tools::get_constructor(base_class_def);

    SgVariableSymbol *initializer_symbol = NULL;

    // First, we analyze the initializer list of the base constructor.
    // we have to figure out which variables initializes the given reference
    for(SgInitializedNamePtrList::iterator
        iter  = ctor_base->get_CtorInitializerList()->get_ctors().begin();
        iter != ctor_base->get_CtorInitializerList()->get_ctors().end();
        iter++) {

      //TODO hack
      if(reference->get_mangled_name() == (*iter)->get_mangled_name()) {

        SgAssignInitializer *assign_init
          = isSgAssignInitializer((*iter)->get_initializer());

        initializer_symbol
          = isSgVarRefExp(assign_init->get_operand())->get_symbol();

        break;
      }

      // replacement for the hack later
      // if(reference == (*iter)->search_for_symbol_from_symbol_table()) {
      //   break;
      // }
    }

    // Second, we determine the position of the initializer in the constructor
    // parameter list
    int parameter_index = 0;
    for(SgInitializedNamePtrList::iterator
        iter  = ctor_base->get_parameterList()->get_args().begin();
        iter != ctor_base->get_parameterList()->get_args().end();
        iter++, parameter_index++) {

      if(initializer_symbol == (*iter)->search_for_symbol_from_symbol_table()) {
        break;
      }
    }

    SgMemberFunctionDeclaration *ctor_parent
      = risc::tools::get_constructor(parent_class_def);

    SgConstructorInitializer *ctor_init = NULL;

    // Next, we have to determine the constructor initializer in the parent
    // class
    for(SgInitializedNamePtrList::iterator
        iter  = ctor_parent->get_CtorInitializerList()->get_ctors().begin();
        iter != ctor_parent->get_CtorInitializerList()->get_ctors().end();
        iter++) {

      //TODO hack
      if(parent_member->get_vardefn()->get_mangled_name()
          == (*iter)->get_mangled_name()) {

        ctor_init = isSgConstructorInitializer((*iter)->get_initptr());
        break;
      }
    }

    // Last, we get the mapped variable
    SgExpression *expr
      = ctor_init->get_args()->get_expressions().at(parameter_index);

    SgVarRefExp *var_ref = NULL;

    SgArrowExp *arrow_exp = isSgArrowExp(expr);
    if(arrow_exp) {
      var_ref = isSgVarRefExp(arrow_exp->get_rhs_operand());
    }

    SgVarRefExp *temp_var_ref = isSgVarRefExp(expr);
    if(temp_var_ref) {
      var_ref = temp_var_ref;
    }

    return var_ref->get_symbol();
  } else {

    SgMemberFunctionDeclaration *ctor_base
      = risc::tools::get_constructor(base_class_def);

    // First, we have to look up the
    int parameter_index = 0;
    for(SgInitializedNamePtrList::iterator
        iter  = ctor_base->get_parameterList()->get_args().begin();
        iter != ctor_base->get_parameterList()->get_args().end();
        iter++, parameter_index++) {

      if(reference == (*iter)->search_for_symbol_from_symbol_table()) {
        break;
      }
    }

    SgMemberFunctionDeclaration *ctor_parent
      = risc::tools::get_constructor(parent_class_def);

    SgConstructorInitializer *ctor_init = NULL;

    // Next, we have to determine the constructor initializer in the parent
    // class
    for(SgInitializedNamePtrList::iterator
        iter  = ctor_parent->get_CtorInitializerList()->get_ctors().begin();
        iter != ctor_parent->get_CtorInitializerList()->get_ctors().end();
        iter++) {

      //TODO hack
      if(parent_member->get_vardefn()->get_mangled_name()
          == (*iter)->get_mangled_name()) {

        ctor_init = isSgConstructorInitializer((*iter)->get_initptr());
        break;
      }
    }

    // Last, we get the mapped variable
    SgExpression *expr
      = ctor_init->get_args()->get_expressions().at(parameter_index);

    SgVarRefExp *var_ref = NULL;

    SgArrowExp *arrow_exp = isSgArrowExp(expr);
    if(arrow_exp) {
      var_ref = isSgVarRefExp(arrow_exp->get_rhs_operand());
    }

    SgVarRefExp *temp_var_ref = isSgVarRefExp(expr);
    if(temp_var_ref) {
      var_ref = temp_var_ref;
    }

    return var_ref->get_symbol();
  }
}

void
risc::InstanceTree::print_instance_tree()
{
  if(path_.empty()) {
    std::cout << "[ ]" << std::endl;

  } else {
    for(std::list<Element>::iterator
        iter  = path_.begin();
        iter != path_.end();) {

      std::cout << "[ ";

      if((*iter).first) {
        std::cout << (*iter).first->get_name();
      } else {
        std::cout << "Global Scope";
      }

      std::cout << " :: "
                << (*iter).second->get_name()
                << " ]";

      if(++iter !=  path_.end()) {
        std::cout << " -> ";
      }
    }
    std::cout << std::endl;
  }
}

risc::InstanceTree
risc::InstanceTree::get_parent()
{
  InstanceTree parent(*this);
  if(!parent.path_.empty()) {

    parent.path_.pop_back();
    return parent;

  } else {

    return parent;
  }
}

std::list<risc::InstanceTree>
risc::InstanceTree::find_all_instances_of(SgVariableDefinition *var_def)
{
  Element e = path_.back();
  std::list<InstanceTree> occurence;

  Instance *inst = dynamic_cast<Instance*>(e.second);
  if(!inst) {

    if(e.second->get_ast_node()->get_vardefn()->
        search_for_symbol_from_symbol_table()
        == var_def->get_vardefn()->search_for_symbol_from_symbol_table()) {

      occurence.push_back(*this);
    }
    return occurence;
  } else {
    recursive_tree_walk(occurence, var_def);
  }
}


void
risc::InstanceTree::recursive_tree_walk(
    std::list<risc::InstanceTree>& occurence, SgVariableDefinition *var_def)
{
  ModuleInstance *module_instance
    = dynamic_cast<ModuleInstance*>(path_.back().second);

  if(module_instance) {

    for(ModuleInstanceVectorIter
        iter  = module_instance->definition_->sub_modules_.begin();
        iter != module_instance->definition_->sub_modules_.end();
        iter++) {

      Element sub_path;

      sub_path.first = module_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      recursive_tree_walk(occurence, var_def );
      path_.pop_back();
    }

    for(ModuleInstanceVectorIter
        iter  = module_instance->definition_->sub_modules_.begin();
        iter != module_instance->definition_->sub_modules_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(PrimitiveChannelInstanceVectorIter
        iter  = module_instance->definition_->sub_prim_channel_.begin();
        iter != module_instance->definition_->sub_prim_channel_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(HierarchicalChannelInstanceVectorIter
        iter  = module_instance->definition_->sub_hier_channel_.begin();
        iter != module_instance->definition_->sub_hier_channel_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(EventVectorIter
        iter  = module_instance->definition_->events_.begin();
        iter != module_instance->definition_->events_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {

        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(VariableVectorIter
        iter  = module_instance->definition_->variables_.begin();
        iter != module_instance->definition_->variables_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(InPortVectorIter
        iter  = module_instance->definition_->inports_.begin();
        iter != module_instance->definition_->inports_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(OutPortVectorIter
        iter  = module_instance->definition_->outports_.begin();
        iter != module_instance->definition_->outports_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(InOutPortVectorIter
        iter  = module_instance->definition_->inoutports_.begin();
        iter != module_instance->definition_->inoutports_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }
  }

  HierarchicalChannelInstance *hier_channel_instance
    = dynamic_cast<HierarchicalChannelInstance*>(path_.back().second);

  if(hier_channel_instance) {

    for(ModuleInstanceVectorIter
        iter  = hier_channel_instance->definition_->sub_modules_.begin();
        iter != hier_channel_instance->definition_->sub_modules_.end();
        iter++) {

      Element sub_path;

      sub_path.first = hier_channel_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      recursive_tree_walk(occurence, var_def );
      path_.pop_back();
    }

    for(ModuleInstanceVectorIter
        iter  = hier_channel_instance->definition_->sub_modules_.begin();
        iter != hier_channel_instance->definition_->sub_modules_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(PrimitiveChannelInstanceVectorIter
        iter  = hier_channel_instance->definition_->sub_prim_channel_.begin();
        iter != hier_channel_instance->definition_->sub_prim_channel_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(HierarchicalChannelInstanceVectorIter
        iter  = hier_channel_instance->definition_->sub_hier_channel_.begin();
        iter != hier_channel_instance->definition_->sub_hier_channel_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(EventVectorIter
        iter  = hier_channel_instance->definition_->events_.begin();
        iter != hier_channel_instance->definition_->events_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(VariableVectorIter
        iter  = hier_channel_instance->definition_->variables_.begin();
        iter != hier_channel_instance->definition_->variables_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(InPortVectorIter
        iter  = hier_channel_instance->definition_->inports_.begin();
        iter != hier_channel_instance->definition_->inports_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(OutPortVectorIter
        iter  = hier_channel_instance->definition_->outports_.begin();
        iter != hier_channel_instance->definition_->outports_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }

    for(InOutPortVectorIter
        iter  = hier_channel_instance->definition_->inoutports_.begin();
        iter != hier_channel_instance->definition_->inoutports_.end();
        iter++) {
      if((*iter)->get_ast_node() == var_def) {
        InstanceTree temp = *this;
        Element e(module_instance->definition_, *iter);
        temp.path_.push_back(e);
        occurence.push_back(temp);
      }
    }
  }
}

risc::Class*
risc::InstanceTree::get_scope()
{
  return path_.back().first;
}

risc::Object*
risc::InstanceTree::get_object()
{
  return path_.back().second;
}

SgVariableSymbol*
risc::InstanceTree::get_object_symbol()
{
  return get_object()->get_symbol();
}

risc::InstanceTree::Element
risc::InstanceTree::pop_back()
{
  Element last = path_.back();
  path_.pop_back();
  return last;
}

void
risc::InstanceTree::push_back(Element ohs)
{
  path_.push_back(ohs);
}

void
risc::InstanceTree::get_all_ports(std::list<InstanceTree>& occurence)
{
  ModuleInstance *module_instance = dynamic_cast<ModuleInstance*>(get_object());

  if(module_instance) {

    for(ModuleInstanceVectorIter
        iter  = module_instance->definition_->sub_modules_.begin();
        iter != module_instance->definition_->sub_modules_.end();
        iter++) {

      Element sub_path;

      sub_path.first = module_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      get_all_ports(occurence);
      path_.pop_back();
    }

    for(HierarchicalChannelInstanceVectorIter
        iter  = module_instance->definition_->sub_hier_channel_.begin();
        iter != module_instance->definition_->sub_hier_channel_.end();
        iter++) {

      Element sub_path;

      sub_path.first = module_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      get_all_ports(occurence);
      path_.pop_back();
    }

    for(InPortVectorIter
        iter  = module_instance->definition_->inports_.begin();
        iter != module_instance->definition_->inports_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(module_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }

    for(OutPortVectorIter
        iter  = module_instance->definition_->outports_.begin();
        iter != module_instance->definition_->outports_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(module_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }

    for(InOutPortVectorIter
        iter  = module_instance->definition_->inoutports_.begin();
        iter != module_instance->definition_->inoutports_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(module_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }
  }


  HierarchicalChannelInstance *hier_channel_instance
    = dynamic_cast<HierarchicalChannelInstance*>(get_object());

  if(hier_channel_instance) {

    for(ModuleInstanceVectorIter
        iter  = hier_channel_instance->definition_->sub_modules_.begin();
        iter != hier_channel_instance->definition_->sub_modules_.end();
        iter++) {

      Element sub_path;

      sub_path.first = hier_channel_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      get_all_ports(occurence);
      path_.pop_back();
    }

    for(HierarchicalChannelInstanceVectorIter
        iter  = hier_channel_instance->definition_->sub_hier_channel_.begin();
        iter != hier_channel_instance->definition_->sub_hier_channel_.end();
        iter++) {

      Element sub_path;

      sub_path.first = hier_channel_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      get_all_ports(occurence);
      path_.pop_back();
    }

    for(InPortVectorIter
        iter  = hier_channel_instance->definition_->inports_.begin();
        iter != hier_channel_instance->definition_->inports_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(hier_channel_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }

    for(OutPortVectorIter
        iter  = hier_channel_instance->definition_->outports_.begin();
        iter != hier_channel_instance->definition_->outports_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(hier_channel_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }

    for(InOutPortVectorIter
        iter  = hier_channel_instance->definition_->inoutports_.begin();
        iter != hier_channel_instance->definition_->inoutports_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(hier_channel_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }
  }

  PrimitiveChannelInstance *prim_channel_instance
    = dynamic_cast<PrimitiveChannelInstance*>(get_object());

  if(prim_channel_instance) {

    for(InPortVectorIter
        iter  = prim_channel_instance->definition_->inports_.begin();
        iter != prim_channel_instance->definition_->inports_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(prim_channel_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }

    for(OutPortVectorIter
        iter  = prim_channel_instance->definition_->outports_.begin();
        iter != prim_channel_instance->definition_->outports_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(prim_channel_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }

    for(InOutPortVectorIter
        iter  = prim_channel_instance->definition_->inoutports_.begin();
        iter != prim_channel_instance->definition_->inoutports_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(prim_channel_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }
  }
}

void
risc::InstanceTree::get_all_events(std::list<InstanceTree>& occurence)
{
  ModuleInstance *module_instance = dynamic_cast<ModuleInstance*>(get_object());

  if(module_instance) {

    for(ModuleInstanceVectorIter
        iter  = module_instance->definition_->sub_modules_.begin();
        iter != module_instance->definition_->sub_modules_.end();
        iter++) {

      Element sub_path;

      sub_path.first = module_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      get_all_ports(occurence);
      path_.pop_back();
    }

    for(HierarchicalChannelInstanceVectorIter
        iter  = module_instance->definition_->sub_hier_channel_.begin();
        iter != module_instance->definition_->sub_hier_channel_.end();
        iter++) {

      Element sub_path;

      sub_path.first = module_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      get_all_ports(occurence);
      path_.pop_back();
    }

    for(EventVectorIter
        iter  = module_instance->definition_->events_.begin();
        iter != module_instance->definition_->events_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(module_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }
  }

  HierarchicalChannelInstance *hier_channel_instance
    = dynamic_cast<HierarchicalChannelInstance*>(get_object());

  if(hier_channel_instance) {

    for(ModuleInstanceVectorIter
        iter  = hier_channel_instance->definition_->sub_modules_.begin();
        iter != hier_channel_instance->definition_->sub_modules_.end();
        iter++) {

      Element sub_path;

      sub_path.first = hier_channel_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      get_all_ports(occurence);
      path_.pop_back();
    }

    for(HierarchicalChannelInstanceVectorIter
        iter  = hier_channel_instance->definition_->sub_hier_channel_.begin();
        iter != hier_channel_instance->definition_->sub_hier_channel_.end();
        iter++) {

      Element sub_path;

      sub_path.first = hier_channel_instance->definition_;
      sub_path.second = *iter;

      path_.push_back(sub_path);
      get_all_ports(occurence);
      path_.pop_back();
    }

    for(EventVectorIter
        iter  = hier_channel_instance->definition_->events_.begin();
        iter != hier_channel_instance->definition_->events_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(hier_channel_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }
  }

  PrimitiveChannelInstance *prim_channel_instance
    = dynamic_cast<PrimitiveChannelInstance*>(get_object());

  if(prim_channel_instance) {

    for(EventVectorIter
        iter  = prim_channel_instance->definition_->events_.begin();
        iter != prim_channel_instance->definition_->events_.end();
        iter++) {

      InstanceTree temp = *this;
      Element e(prim_channel_instance->definition_, *iter);
      temp.path_.push_back(e);
      occurence.push_back(temp);
    }
  }
}

bool
risc::InstanceTree::operator==(const InstanceTree &ohs)
{
  if(path_.size() != ohs.path_.size()) {
    return false;
  }

  Path::const_iterator this_path_iter = path_.begin();
  Path::const_iterator ohs_path_iter = ohs.path_.begin();;

  for(; this_path_iter != path_.end(); this_path_iter++, ohs_path_iter++) {

    if(((*this_path_iter).first != (*ohs_path_iter).first) ||
        ((*this_path_iter).second != (*ohs_path_iter).second)) {
      return false;
    }
  }

  return true;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
