#ifndef PATH_INSTANCE_MAPPER_H_INCLUDED_
#define PATH_INSTANCE_MAPPER_H_INCLUDED_

#include<vector>
#include<utility>

#include "../segment_graph/segment_graph.h"
#include "instance_tree.h"
#include "module.h"
#include "module_instance.h"
#include "hierarchical_channel.h"
#include "hierarchical_channel_instance.h"

namespace risc {

class Design;

/**
 * \class PathInstanceMapper
 * \brief This class can be used as a lookup table between an instance id of
 *        a module or hierarchical channel and the instance tree
 */
class PathInstanceMapper {

public:

  /**
   * \function PathInstanceMapper
   * \brief    This is the default constructor and need a design and a segment
   *           graph. The lookup table will be automatically created.
   */
  PathInstanceMapper();

  /**
   * \function determine_lookup_table
   * \brief    This function traverses the design and pushes all modules and
   *           hierarchical channel instances into the path_to_id_mapping_
   *           lookup table. The traversing is dependent on the declaration oder
   *           of the member variables. Also, it follows depth first search.
   *           CAUTION: The function Design::reset_instance_counter() must be
   *                    used first! Otherwise wrong intance id's a generated!
   */
  void determine_lookup_table(InstanceTree root);

  /**
   * \function print_mapping
   * \brief    The function prints the lookup table in the terminal
   */
  void print_mapping();

  /**
   * \function get_id
   * \brief    This function returns for a given tree the corresponding instance
   *           id.
   */
  int get_id(InstanceTree tree);

  /**
   * \function get_instance_Tree
   * \brief    This function returns for a given id and potential mapping
   *           candidates the corresponding instance tree.
   */
  InstanceTree
  get_instance_tree(int id, std::list<InstanceTree> potential_instances);

  typedef std::pair<InstanceTree, int> PathWithID;

  /**
   * \brief This is the lookup table from path to instance id
   */
  std::vector<PathWithID> path_to_id_mapping_;

  /**
   * \brief This variable counts what is the max of instances
   */
  int max_instances_;
};

}; // end of namespace risc


#endif /* PATH_INSTANCE_MAPPER_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
