#include "graph_helper.h"

std::list<int>
risc::sg::get_all_reachable_segments(SegmentGraph &segment_graph,
    int starting_id)
{
  std::pair<VertexIterator, VertexIterator> vp;

  VertexIterator starting_vertex;

  // Here we are searching for the for the segment with the id 'starting_id'
  for(vp = boost::vertices(segment_graph.graph_);
      vp.first != vp.second; ++vp.first) {

    if(segment_graph.graph_[*(vp.first)].id_ == starting_id) {
      starting_vertex = vp.first;
      break;
    }
  }

  // Reseting all segments to the status untouched
  segment_graph.set_all_segments_to_untouched();

  risc::sg::SegmentGraph::SegmentSet untouched;
  untouched.insert(*starting_vertex);

  std::list<int> reachable_segment_ids;

  // We are iterating until there are no more untouched segments
  while(!untouched.empty()) {

    VertexDescriptor vertex = *untouched.begin();

    // Marking the current segment as touched
    segment_graph.graph_[vertex].helper_for_graph_algorithms_
      = Segment::Touched;

    // Adding the id to the return list of reachable segment ids
    reachable_segment_ids.push_back(segment_graph.graph_[vertex].id_);

    Graph::out_edge_iterator out_edge_begin, out_edge_end, out_edge_iter;
    boost::tie(out_edge_begin, out_edge_end)
      = boost::out_edges(vertex, segment_graph.graph_);

    // Iterating over all outgoing edges to discover new segments
    for(out_edge_iter  = out_edge_begin;
        out_edge_iter != out_edge_end;
        out_edge_iter++) {

      VertexDescriptor target
        = boost::target(*out_edge_iter, segment_graph.graph_);

      // If the segment is untouched, we have to add it to the set of untouched
      // segments. We will evaluate this segment in one of the next iterations.
      if(segment_graph.graph_[target].helper_for_graph_algorithms_
          == Segment::Untouched) {

        untouched.insert(target);
      }
    }

    // Removing the touched segment from the untouched segment set
    untouched.erase(vertex);
  }

  return reachable_segment_ids;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
