#include "time.h"

risc::sg::Time::Time():
  time_units_(0),
  delta_count_(0),
  infinite_(false)
{ }

risc::sg::Time::Time(long long time_units, int delta_count):
  time_units_(time_units),
  delta_count_(delta_count),
  infinite_(false)
{ }

risc::sg::Time::Time(bool infinite):
  time_units_(0),
  delta_count_(0),
  infinite_(infinite)
{ }

bool
risc::sg::Time::operator<(const Time &ohs)
{
  if(time_units_ < ohs.time_units_) {
    return true;
  }

  if(time_units_ == ohs.time_units_ &&
      delta_count_ < ohs.delta_count_) {
    return true;
  }

  return false;
}

bool
risc::sg::Time::operator>(const Time &ohs)
{
  if(time_units_ > ohs.time_units_) {
    return true;
  }

  if(time_units_ == ohs.time_units_ &&
      delta_count_ > ohs.delta_count_) {
    return true;
  }

  return false;
}

bool
risc::sg::Time::operator<=(const Time &ohs)
{
  return (*this < ohs) || (*this == ohs);
}

bool
risc::sg::Time::operator>=(const Time &ohs)
{
  return (*this > ohs) || (*this == ohs);
}

bool
risc::sg::Time::operator==(const Time &ohs)
{
  return time_units_ == ohs.time_units_ &&
    delta_count_ == ohs.delta_count_;
}

bool
risc::sg::Time::operator!=(const Time &ohs)
{
  return !(*this == ohs);
}

risc::sg::Time
risc::sg::Time::operator+(const Time &ohs)
{
  // This part needs more discussion
  if(time_units_ == 0 && ohs.time_units_ == 0) {

    return Time(0, delta_count_ + ohs.delta_count_);
  } else {

    return Time(time_units_ + ohs.time_units_, 0);
  }
}

risc::sg::Time&
risc::sg::Time::operator+=(const Time &ohs)
{
  *this = *this + ohs;
  return *this;
}

bool
risc::sg::Time::one_delta_cycle_ahead(const Time &ohs)
{
  return (time_units_ == ohs.time_units_)
    && (delta_count_ == (ohs.delta_count_ - 1));
}

std::ostream&
risc::sg::operator<<(std::ostream& os, const risc::sg::Time& time)
{
  if(time.infinite_) {

    os << "(&#8734;:&#8734;)";
  } else {
    os << "(" << time.time_units_ << ":" << time.delta_count_ << ")";
  }
  return os;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
