#ifndef COMMAND_LINE_PARSING_H_INCLUDED_
#define COMMAND_LINE_PARSING_H_INCLUDED_

#include "rose.h"
#include "commandline_processing.h"

namespace risc {

namespace tools {


/**
  * \brief      This function takes the parameter of the command line and
  *             creates a source file which includes all the source files.
  *             Afterwards all source files are replaced with new file in the
  *             command parameters. Example
  *
  *             Input:
  *               ./myprogram file_a.cpp file_b.cpp file_c.cpp
  *
  *             Output
  *               ./myprogram __main__.cpp
  *
  *             New file __main__.cpp
  *               #include "file_a.cpp"
  *               #include "file_b.cpp"
  *               #include "file_c.cpp"
  *
  * \return     vector of strings with the new parameters
  */
std::vector<std::string>
preparsing_commandline_arguments(int &argc, char **&argv,
    std::vector<std::string> &input_files);


} // end namespace tools

} // end namespace risc

#endif /* COMMAND_LINE_PARSING_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
