#include "systemc_definition_finder.h"
#include "../tools/search_class_definition_by_name.h"


risc::tools::SystemCDefinitionFinder*
risc::tools::SystemCDefinitionFinder::instance = NULL;

risc::tools::SystemCDefinitionFinder*
risc::tools::SystemCDefinitionFinder::get_definitions(SgProject *project)
{
  if(instance) {
    return instance;
  } else {
    instance = new SystemCDefinitionFinder(project);
    instance->init_systemc_definitions();
    return instance;
  }
}

void
risc::tools::SystemCDefinitionFinder::init_systemc_definitions()
{
  sc_module_       = risc::tools::find_class_definition(
                         "sc_module","::sc_core::",project_);

  sc_port_         = risc::tools::find_class_definition(
                         "sc_port","::sc_core::",project_);

  sc_port_base_    = risc::tools::find_class_definition(
                         "sc_port_base","::sc_core::",project_);

  sc_in_           = risc::tools::find_class_definition(
                         "sc_in","::sc_core::",project_);

  sc_inout_        = risc::tools::find_class_definition(
                         "sc_inout","::sc_core::",project_);

  sc_out_          = risc::tools::find_class_definition(
                         "sc_out","::sc_core::",project_);

  sc_event_        = risc::tools::find_class_definition(
                         "sc_event","::sc_core::",project_);

  sc_prim_channel_ = risc::tools::find_class_definition(
                         "sc_prim_channel","::sc_core::",project_);

  // This is a hack! Why? All four types are in the original library typedefs.
  // However, rose has some hiccups with parsing sc_out. There is no definition
  // available. So, I introduced in the SystemC library some 'face' classes
  // and replaced the typedefs.
  sc_channel_   = risc::tools::find_class_definition(
                      "sc_channel","::sc_core::",project_);

  sc_in_clk_    = risc::tools::find_class_definition(
                      "sc_in_clk","::sc_core::",project_);

  sc_out_clk_   = risc::tools::find_class_definition(
                      "sc_out_clk","::sc_core::",project_);

  sc_inout_clk_ = risc::tools::find_class_definition(
                      "sc_inout_clk","::sc_core::",project_);

/*
  std::cout << "sc_module_        " << sc_module_ << std::endl;
  std::cout << "sc_port_          " << sc_port_  << std::endl;
  std::cout << "sc_port_base_     " << sc_port_base_ << std::endl;
  std::cout << "sc_in_            " << sc_in_ << std::endl;
  std::cout << "sc_inout_         " << sc_inout_ << std::endl;
  std::cout << "sc_out_           " << sc_out_ << std::endl;
  std::cout << "sc_event_         " << sc_event_ << std::endl;
  std::cout << "sc_prim_channel_  " << sc_prim_channel_ << std::endl;
  std::cout << "sc_channel_       " << sc_channel_ << std::endl;
  std::cout << "sc_in_clk_        " << sc_in_clk_ << std::endl;
  std::cout << "sc_out_clk_       " << sc_out_clk_ << std::endl;
  std::cout << "sc_inout_clk_     " << sc_inout_clk_ << std::endl;
*/

  // Search for the SystemC Namespaces sc_core and sc_dt
  Rose_STL_Container<SgNode*> temp_namespaces
    = NodeQuery::querySubTree(project_, V_SgNamespaceDefinitionStatement);

  for(Rose_STL_Container<SgNode*>::iterator
      iter  = temp_namespaces.begin();
      iter != temp_namespaces.end();
      iter++) {

    if(SageInterface::get_name(*iter) == "sc_core") {
      sc_core_ = isSgNamespaceDefinitionStatement(*iter);
    }

    if(SageInterface::get_name(*iter) == "sc_dt") {
      sc_dt_ = isSgNamespaceDefinitionStatement(*iter);
    }
  }

  // Search for all wait functions
  Rose_STL_Container<SgNode*> temp_wait_func_def
    = NodeQuery::querySubTree(project_, V_SgFunctionDefinition);

  for(Rose_STL_Container<SgNode*>::iterator
      iter = temp_wait_func_def.begin();
      iter != temp_wait_func_def.end();
      iter++) {

    SgFunctionDefinition *func_def = isSgFunctionDefinition(*iter);
    SgFunctionDeclaration *func_decl = func_def->get_declaration();

    if(SageInterface::getEnclosingClassDefinition(func_decl) == NULL)
      continue;


    if(SageInterface::get_name(func_decl) == "wait") {

      if(((SageInterface::enclosingNamespaceScope(func_decl))->
            get_namespaceDeclaration())->search_for_symbol_from_symbol_table()
          == (sc_core_->get_namespaceDeclaration())->
            search_for_symbol_from_symbol_table()) {
        sc_wait_funcs_.insert(func_def);
      }
    }
  }
}

risc::tools::SystemCDefinitionFinder::SystemCDefinitionFinder(SgProject *project):
  project_(project)
{

}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
